@@ -1,14 +1,14 @@
 ##
 ##
 twophase<-function(id,strata=NULL, probs=NULL, weights=NULL, fpc=NULL,
-                   subset, data, method=c("full","approx","simple")){
+                   subset, data, method=c("full","approx","simple"), pps=NULL){
 
   data<-detibble(data)
     
   method<-match.arg(method)
   if(method=="full") {
     if (!is.null(weights)) stop("weights not accepted by method='full'")
-    return(twophase2(id=id, strata=strata, probs=probs, fpc=fpc,subset=subset,data=data))
+    return(twophase2(id=id, strata=strata, probs=probs, fpc=fpc,subset=subset,data=data,pps=pps))
   }
                      
   d1<-svydesign(ids=id[[1]],strata=strata[[1]],weights=weights[[1]],
@@ -93,9 +93,9 @@ twophase<-function(id,strata=NULL, probs=NULL, weights=NULL, fpc=NULL,
       cm<-rval$phase1$full$cluster[,m]
       if (nunique(sa)!=nunique(sa[subset]))
           stop("Some phase-2 strata have zero sampling fraction")
-      rval$usu<-ave(cm[subset],sa[subset],FUN=nunique)/ave(cm,sa,FUN=nunique)[subset]
+      rval$usu<-ave(as.numeric(cm[subset]),sa[subset],FUN=nunique)/ave(as.numeric(cm),sa,FUN=nunique)[subset]
   } else {
-      rval$usu<-drop(with(rval$phase1$sample,ave(cluster[,m], strata[,m], FUN=nunique))/rval$phase1$full$fpc$sampsize[rval$subset])
+      rval$usu<-drop(with(rval$phase1$sample,ave(as.numeric(cluster[,m]), strata[,m], FUN=nunique))/rval$phase1$full$fpc$sampsize[rval$subset,m])
   }
 
 ##  if (any(rval$usu<1) && any(duplicated(d1$cluster[,1])))
@@ -699,7 +699,7 @@ estWeights.twophase<-function(data, formula=NULL, working.model=NULL,...){
     model<-glm(ff, data=data$phase1$full$variables, family=binomial(),
                subset=!certainty, na.action=na.fail)
   } else {
-    xx<-estfun(working.model)
+    xx<-estfuns(working.model)
     model<-glm(data$subset~xx,family=binomial(), subset=!certainty, na.action=na.fail)
   }
   fitp<-as.numeric(certainty[data$subset])
@@ -739,13 +739,13 @@ estWeights.twophase<-function(data, formula=NULL, working.model=NULL,...){
 }
 
 
-estfun<-function(model,...) UseMethod("estfun")
-estfun.coxph<-function(model, ...) resid(model,"score")
-estfun.glm<-function(model){
+estfuns<-function(model,...) UseMethod("estfuns")
+estfuns.coxph<-function(model, ...) resid(model,"score")
+estfuns.glm<-function(model,...){
   xmat<-model.matrix(model)
   residuals(model,"working")*model$weights*xmat
 }
-estfun.lm<-function(model,...){
+estfuns.lm<-function(model,...){
   model.matrix(model)*resid(model)
 }
 
